#ifndef POKEPLATINUM_UNK_02002F38_H
#define POKEPLATINUM_UNK_02002F38_H

#include "struct_decls/struct_02002F38_decl.h"

UnkStruct_02002F38 * sub_02002F38(int param0);
void sub_02002F54(UnkStruct_02002F38 * param0);
void sub_02002F5C(UnkStruct_02002F38 * param0, int param1, void * param2, void * param3, u32 param4);
void sub_02002F70(UnkStruct_02002F38 * param0, int param1, u32 param2, u32 param3);
void sub_02002FA0(UnkStruct_02002F38 * param0, int param1);
void sub_02002FBC(UnkStruct_02002F38 * param0, const void * param1, int param2, u16 param3, u16 param4);
void sub_02002FEC(UnkStruct_02002F38 * param0, u32 param1, u32 param2, u32 param3, int param4, u32 param5, u16 param6, u16 param7);
void sub_02003050(UnkStruct_02002F38 * param0, u32 param1, u32 param2, u32 param3, int param4, u32 param5, u16 param6);
void sub_02003070(UnkStruct_02002F38 * param0, int param1, u16 param2, u32 param3);
void sub_020030E4(u32 param0, u32 param1, u32 param2, u32 param3, u16 param4, void * param5);
void sub_02003120(UnkStruct_02002F38 * param0, int param1, u16 param2, int param3, u16 param4, u16 param5);
u16 * sub_02003164(UnkStruct_02002F38 * param0, int param1);
u16 * sub_0200316C(UnkStruct_02002F38 * param0, int param1);
u8 sub_02003178(UnkStruct_02002F38 * param0, u16 param1, u16 param2, s8 param3, u8 param4, u8 param5, u16 param6);
void sub_02003694(UnkStruct_02002F38 * param0);
u16 sub_0200384C(UnkStruct_02002F38 * param0);
void sub_02003858(UnkStruct_02002F38 * param0, int param1);
void sub_0200387C(UnkStruct_02002F38 * param0, u8 param1);
void sub_020038B0(UnkStruct_02002F38 * param0, int param1, int param2, u16 param3, u16 param4, u16 param5);
u16 sub_02003910(UnkStruct_02002F38 * param0, int param1, int param2, u16 param3);
void sub_0200393C(const u16 * param0, u16 * param1, u16 param2, u8 param3, u16 param4);
void sub_020039B0(UnkStruct_02002F38 * param0, int param1, u16 param2, u16 param3, u8 param4, u16 param5);
void sub_020039F8(const u16 * param0, u16 * param1, u16 param2, u8 param3, u16 param4);
void sub_02003A2C(UnkStruct_02002F38 * param0, int param1, u16 param2, u8 param3, u16 param4);
void sub_02003A8C(u16 * param0, int param1, int param2, int param3, int param4);
void sub_02003B08(UnkStruct_02002F38 * param0, u32 param1, u32 param2, u32 param3, int param4, u32 param5, u16 param6, int param7, int param8, int param9);

#endif // POKEPLATINUM_UNK_02002F38_H
